@@ -33,7 +33,7 @@ var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: tru
 
 // node_modules/color-name/index.js
 var require_color_name = __commonJS({
-  "node_modules/color-name/index.js"(exports, module2) {
+  "node_modules/color-name/index.js"(exports2, module2) {
     "use strict";
     module2.exports = {
       "aliceblue": [240, 248, 255],
@@ -190,7 +190,7 @@ var require_color_name = __commonJS({
 
 // node_modules/color-convert/conversions.js
 var require_conversions = __commonJS({
-  "node_modules/color-convert/conversions.js"(exports, module2) {
+  "node_modules/color-convert/conversions.js"(exports2, module2) {
     var cssKeywords = require_color_name();
     var reverseKeywords = {};
     for (const key of Object.keys(cssKeywords)) {
@@ -861,7 +861,7 @@ var require_conversions = __commonJS({
 
 // node_modules/color-convert/route.js
 var require_route = __commonJS({
-  "node_modules/color-convert/route.js"(exports, module2) {
+  "node_modules/color-convert/route.js"(exports2, module2) {
     var conversions = require_conversions();
     function buildGraph() {
       const graph = {};
@@ -931,7 +931,7 @@ var require_route = __commonJS({
 
 // node_modules/color-convert/index.js
 var require_color_convert = __commonJS({
-  "node_modules/color-convert/index.js"(exports, module2) {
+  "node_modules/color-convert/index.js"(exports2, module2) {
     var conversions = require_conversions();
     var route = require_route();
     var convert = {};
@@ -992,7 +992,7 @@ var require_color_convert = __commonJS({
 
 // node_modules/ansi-styles/index.js
 var require_ansi_styles = __commonJS({
-  "node_modules/ansi-styles/index.js"(exports, module2) {
+  "node_modules/ansi-styles/index.js"(exports2, module2) {
     "use strict";
     var wrapAnsi16 = (fn, offset) => (...args) => {
       const code = fn(...args);
@@ -1134,7 +1134,7 @@ var require_ansi_styles = __commonJS({
 
 // node_modules/has-flag/index.js
 var require_has_flag = __commonJS({
-  "node_modules/has-flag/index.js"(exports, module2) {
+  "node_modules/has-flag/index.js"(exports2, module2) {
     "use strict";
     module2.exports = (flag, argv = process.argv) => {
       const prefix = flag.startsWith("-") ? "" : flag.length === 1 ? "-" : "--";
@@ -1147,7 +1147,7 @@ var require_has_flag = __commonJS({
 
 // node_modules/supports-color/index.js
 var require_supports_color = __commonJS({
-  "node_modules/supports-color/index.js"(exports, module2) {
+  "node_modules/supports-color/index.js"(exports2, module2) {
     "use strict";
     var os = require("os");
     var tty = require("tty");
@@ -1249,7 +1249,7 @@ var require_supports_color = __commonJS({
 
 // node_modules/chalk/source/util.js
 var require_util = __commonJS({
-  "node_modules/chalk/source/util.js"(exports, module2) {
+  "node_modules/chalk/source/util.js"(exports2, module2) {
     "use strict";
     var stringReplaceAll = (string, substring, replacer) => {
       let index = string.indexOf(substring);
@@ -1288,7 +1288,7 @@ var require_util = __commonJS({
 
 // node_modules/chalk/source/templates.js
 var require_templates = __commonJS({
-  "node_modules/chalk/source/templates.js"(exports, module2) {
+  "node_modules/chalk/source/templates.js"(exports2, module2) {
     "use strict";
     var TEMPLATE_REGEX = /(?:\\(u(?:[a-f\d]{4}|\{[a-f\d]{1,6}\})|x[a-f\d]{2}|.))|(?:\{(~)?(\w+(?:\([^)]*\))?(?:\.\w+(?:\([^)]*\))?)*)(?:[ \t]|(?=\r?\n)))|(\})|((?:.|[\r\n\f])+?)/gi;
     var STYLE_REGEX = /(?:^|\.)(\w+)(?:\(([^)]*)\))?/g;
@@ -1402,7 +1402,7 @@ var require_templates = __commonJS({
 
 // node_modules/chalk/source/index.js
 var require_source = __commonJS({
-  "node_modules/chalk/source/index.js"(exports, module2) {
+  "node_modules/chalk/source/index.js"(exports2, module2) {
     "use strict";
     var ansiStyles = require_ansi_styles();
     var { stdout: stdoutColor, stderr: stderrColor } = require_supports_color();
@@ -1622,7 +1622,16 @@ function info(...text) {
 // src/task-runtime.ts
 var ENV_TRIM_LEN = 20;
 var ARGS_MARKER = "$@";
-var _TaskRuntime = class {
+var TaskRuntime = class _TaskRuntime {
+  static {
+    /**
+     * The project-relative path of the tasks manifest file.
+     */
+    this.MANIFEST_FILE = path.posix.join(
+      PROJEN_DIR,
+      "tasks.json"
+    );
+  }
   constructor(workdir) {
     this.workdir = (0, import_path.resolve)(workdir);
     const manifestPath = (0, import_path.join)(this.workdir, _TaskRuntime.MANIFEST_FILE);
@@ -1655,14 +1664,6 @@ var _TaskRuntime = class {
     new RunTask(this, task, parents, args);
   }
 };
-var TaskRuntime = _TaskRuntime;
-/**
- * The project-relative path of the tasks manifest file.
- */
-TaskRuntime.MANIFEST_FILE = path.posix.join(
-  PROJEN_DIR,
-  "tasks.json"
-);
 var RunTask = class {
   constructor(runtime, task, parents = [], args = []) {
     this.runtime = runtime;
@@ -1701,6 +1702,14 @@ var RunTask = class {
       );
     }
     for (const step of task.steps) {
+      if (!this.evalCondition(step)) {
+        this.log("condition exited with non-zero - skipping");
+        continue;
+      }
+      const argsList = [
+        ...step.args || [],
+        ...step.receiveArgs ? args : []
+      ].map((a) => a.toString());
       if (step.say) {
         info(this.fmtLog(step.say));
       }
@@ -1708,10 +1717,11 @@ var RunTask = class {
         this.runtime.runTask(
           step.spawn,
           [...this.parents, this.task.name],
-          step.receiveArgs ? args : []
+          argsList
         );
       }
       const execs = step.exec ? [step.exec] : [];
+      const env = this.evalEnvironment(step.env ?? {});
       if (step.builtin) {
         execs.push(this.renderBuiltin(step.builtin));
       }
@@ -1724,18 +1734,17 @@ var RunTask = class {
         } else {
           command = exec;
         }
-        if (step.receiveArgs) {
-          if (command.includes(ARGS_MARKER)) {
-            command = command.replace(ARGS_MARKER, args.join(" "));
-          } else {
-            command = [command, ...args].join(" ");
-          }
+        if (command.includes(ARGS_MARKER)) {
+          command = command.replace(ARGS_MARKER, argsList.join(" "));
+        } else {
+          command = [command, ...argsList].join(" ");
         }
         const cwd = step.cwd;
         try {
           const result = this.shell({
             command,
-            cwd
+            cwd,
+            extraEnv: env
           });
           hasError = result.status !== 0;
         } catch (e) {
@@ -1761,13 +1770,13 @@ var RunTask = class {
    * evaluates to false (exits with non-zero), indicating that the task should
    * be skipped.
    */
-  evalCondition(task) {
-    if (!task.condition) {
+  evalCondition(taskOrStep) {
+    if (!taskOrStep.condition) {
       return true;
     }
-    this.log((0, import_chalk.gray)(`${(0, import_chalk.underline)("condition")}: ${task.condition}`));
+    this.log((0, import_chalk.gray)(`${(0, import_chalk.underline)("condition")}: ${taskOrStep.condition}`));
     const result = this.shell({
-      command: task.condition,
+      command: taskOrStep.condition,
       logprefix: "condition: ",
       quiet: true
     });
@@ -1777,6 +1786,28 @@ var RunTask = class {
       return false;
     }
   }
+  /**
+   * Evaluates environment variables from shell commands (e.g. `$(xx)`)
+   */
+  evalEnvironment(env) {
+    const output = {};
+    for (const [key, value] of Object.entries(env ?? {})) {
+      if (String(value).startsWith("$(") && String(value).endsWith(")")) {
+        const query = value.substring(2, value.length - 1);
+        const result = this.shellEval({ command: query });
+        if (result.status !== 0) {
+          const error = result.error ? result.error.stack : result.stderr?.toString() ?? "unknown error";
+          throw new Error(
+            `unable to evaluate environment variable ${key}=${value}: ${error}`
+          );
+        }
+        output[key] = result.stdout.toString("utf-8").trim();
+      } else {
+        output[key] = value;
+      }
+    }
+    return output;
+  }
   /**
    * Renders the runtime environment for a task. Namely, it supports this syntax
    * `$(xx)` for allowing environment to be evaluated by executing a shell
@@ -1794,23 +1825,7 @@ var RunTask = class {
       ...env,
       ...this.task.env ?? {}
     };
-    const output = {};
-    for (const [key, value] of Object.entries(env ?? {})) {
-      if (String(value).startsWith("$(") && String(value).endsWith(")")) {
-        const query = value.substring(2, value.length - 1);
-        const result = this.shellEval({ command: query });
-        if (result.status !== 0) {
-          const error = result.error ? result.error.stack : result.stderr?.toString() ?? "unknown error";
-          throw new Error(
-            `unable to evaluate environment variable ${key}=${value}: ${error}`
-          );
-        }
-        output[key] = result.stdout.toString("utf-8").trim();
-      } else {
-        output[key] = value;
-      }
-    }
-    return output;
+    return this.evalEnvironment(env ?? {});
   }
   /**
    * Returns the "full name" of the task which includes all it's parent task names concatenated by chevrons.
@@ -1853,7 +1868,8 @@ var RunTask = class {
       stdio: "inherit",
       env: {
         ...process.env,
-        ...this.env
+        ...this.env,
+        ...options.extraEnv
       },
       ...options.spawnOptions
     });
@@ -1870,7 +1886,7 @@ var RunTask = class {
   renderBuiltin(builtin) {
     const moduleRoot = (0, import_path.dirname)(require.resolve("../package.json"));
     const program = require.resolve((0, import_path.join)(moduleRoot, "lib", `${builtin}.task.js`));
-    return `${process.execPath} ${program}`;
+    return `'${process.execPath}' '${program}'`;
   }
 };
 // Annotate the CommonJS export names for ESM import in node:
