@@ -17,6 +17,7 @@ from .. import (
     DevEnvironmentDockerImage as _DevEnvironmentDockerImage_4a8d8ffd,
     DevEnvironmentOptions as _DevEnvironmentOptions_b10d89d1,
     IDevEnvironment as _IDevEnvironment_9a084622,
+    JsonFile as _JsonFile_fa8164db,
     Project as _Project_57d89203,
     Task as _Task_9fa875b6,
 )
@@ -43,112 +44,67 @@ class Console(enum.Enum):
     '''
 
 
-@jsii.implements(_IDevEnvironment_9a084622)
-class DevContainer(
-    _Component_2b0ad27f,
-    metaclass=jsii.JSIIMeta,
-    jsii_type="projen.vscode.DevContainer",
-):
-    '''(experimental) A development environment running VSCode in a container;
-
-    used by GitHub
-    codespaces.
-
-    :stability: experimental
-    '''
-
+@jsii.data_type(
+    jsii_type="projen.vscode.DevContainerFeature",
+    jsii_struct_bases=[],
+    name_mapping={"name": "name", "version": "version"},
+)
+class DevContainerFeature:
     def __init__(
         self,
-        project: _Project_57d89203,
         *,
-        docker_image: typing.Optional[_DevEnvironmentDockerImage_4a8d8ffd] = None,
-        ports: typing.Optional[typing.Sequence[builtins.str]] = None,
-        tasks: typing.Optional[typing.Sequence[_Task_9fa875b6]] = None,
-        vscode_extensions: typing.Optional[typing.Sequence[builtins.str]] = None,
+        name: builtins.str,
+        version: typing.Optional[builtins.str] = None,
     ) -> None:
-        '''
-        :param project: -
-        :param docker_image: (experimental) A Docker image or Dockerfile for the container.
-        :param ports: (experimental) An array of ports that should be exposed from the container.
-        :param tasks: (experimental) An array of tasks that should be run when the container starts.
-        :param vscode_extensions: (experimental) An array of extension IDs that specify the extensions that should be installed inside the container when it is created.
+        '''(experimental) devcontainer features options.
 
+        :param name: (experimental) feature name.
+        :param version: (experimental) feature version. Default: latest
+
+        :see: https://containers.dev/implementors/features/#devcontainer-json-properties
         :stability: experimental
         '''
         if __debug__:
-            type_hints = typing.get_type_hints(_typecheckingstub__50747b0b39db339de569d0b44aea84c4de3c0d9d7b268479f1a3852dfb712c0b)
-            check_type(argname="argument project", value=project, expected_type=type_hints["project"])
-        options = DevContainerOptions(
-            docker_image=docker_image,
-            ports=ports,
-            tasks=tasks,
-            vscode_extensions=vscode_extensions,
-        )
-
-        jsii.create(self.__class__, self, [project, options])
-
-    @jsii.member(jsii_name="addDockerImage")
-    def add_docker_image(self, image: _DevEnvironmentDockerImage_4a8d8ffd) -> None:
-        '''(experimental) Add a custom Docker image or Dockerfile for the container.
-
-        :param image: -
-
-        :stability: experimental
-        '''
-        if __debug__:
-            type_hints = typing.get_type_hints(_typecheckingstub__24cf7d35da522ed5017c6db46a2565a5b5a09fbffc1dafabdf917aa798fea91d)
-            check_type(argname="argument image", value=image, expected_type=type_hints["image"])
-        return typing.cast(None, jsii.invoke(self, "addDockerImage", [image]))
-
-    @jsii.member(jsii_name="addPorts")
-    def add_ports(self, *ports: builtins.str) -> None:
-        '''(experimental) Adds ports that should be exposed (forwarded) from the container.
-
-        :param ports: The new ports.
-
-        :stability: experimental
-        '''
-        if __debug__:
-            type_hints = typing.get_type_hints(_typecheckingstub__3898b547dc05a8cd9427e24743e3ec87b3108cfb90befb94e77c24e156c3559d)
-            check_type(argname="argument ports", value=ports, expected_type=typing.Tuple[type_hints["ports"], ...]) # pyright: ignore [reportGeneralTypeIssues]
-        return typing.cast(None, jsii.invoke(self, "addPorts", [*ports]))
-
-    @jsii.member(jsii_name="addTasks")
-    def add_tasks(self, *tasks: _Task_9fa875b6) -> None:
-        '''(experimental) Adds tasks to run when the container starts.
-
-        Tasks will be run in sequence.
-
-        :param tasks: The new tasks.
-
-        :stability: experimental
-        '''
-        if __debug__:
-            type_hints = typing.get_type_hints(_typecheckingstub__4eccb3fb9f6eca40de5ffa59e4b75a85830e1113729bd9b08dc7e4128c2bfde0)
-            check_type(argname="argument tasks", value=tasks, expected_type=typing.Tuple[type_hints["tasks"], ...]) # pyright: ignore [reportGeneralTypeIssues]
-        return typing.cast(None, jsii.invoke(self, "addTasks", [*tasks]))
-
-    @jsii.member(jsii_name="addVscodeExtensions")
-    def add_vscode_extensions(self, *extensions: builtins.str) -> None:
-        '''(experimental) Adds a list of VSCode extensions that should be automatically installed in the container.
-
-        :param extensions: The extension IDs.
-
-        :stability: experimental
-        '''
-        if __debug__:
-            type_hints = typing.get_type_hints(_typecheckingstub__c13b0cafe05fa1eb28182ec3465e43421a4aa7c7e44e035db34dd4cdbce24537)
-            check_type(argname="argument extensions", value=extensions, expected_type=typing.Tuple[type_hints["extensions"], ...]) # pyright: ignore [reportGeneralTypeIssues]
-        return typing.cast(None, jsii.invoke(self, "addVscodeExtensions", [*extensions]))
+            type_hints = typing.get_type_hints(_typecheckingstub__5a7f73b2b024ecc62d8491484836d55c16fc65fe902d48c89e27920fb1de29e7)
+            check_type(argname="argument name", value=name, expected_type=type_hints["name"])
+            check_type(argname="argument version", value=version, expected_type=type_hints["version"])
+        self._values: typing.Dict[builtins.str, typing.Any] = {
+            "name": name,
+        }
+        if version is not None:
+            self._values["version"] = version
 
     @builtins.property
-    @jsii.member(jsii_name="config")
-    def config(self) -> typing.Any:
-        '''(experimental) Direct access to the devcontainer configuration (escape hatch).
+    def name(self) -> builtins.str:
+        '''(experimental) feature name.
 
         :stability: experimental
         '''
-        return typing.cast(typing.Any, jsii.get(self, "config"))
+        result = self._values.get("name")
+        assert result is not None, "Required property 'name' is missing"
+        return typing.cast(builtins.str, result)
+
+    @builtins.property
+    def version(self) -> typing.Optional[builtins.str]:
+        '''(experimental) feature version.
+
+        :default: latest
+
+        :stability: experimental
+        '''
+        result = self._values.get("version")
+        return typing.cast(typing.Optional[builtins.str], result)
+
+    def __eq__(self, rhs: typing.Any) -> builtins.bool:
+        return isinstance(rhs, self.__class__) and rhs._values == self._values
+
+    def __ne__(self, rhs: typing.Any) -> builtins.bool:
+        return not (rhs == self)
+
+    def __repr__(self) -> str:
+        return "DevContainerFeature(%s)" % ", ".join(
+            k + "=" + repr(v) for k, v in self._values.items()
+        )
 
 
 @jsii.data_type(
@@ -159,6 +115,7 @@ class DevContainer(
         "ports": "ports",
         "tasks": "tasks",
         "vscode_extensions": "vscodeExtensions",
+        "features": "features",
     },
 )
 class DevContainerOptions(_DevEnvironmentOptions_b10d89d1):
@@ -169,6 +126,7 @@ class DevContainerOptions(_DevEnvironmentOptions_b10d89d1):
         ports: typing.Optional[typing.Sequence[builtins.str]] = None,
         tasks: typing.Optional[typing.Sequence[_Task_9fa875b6]] = None,
         vscode_extensions: typing.Optional[typing.Sequence[builtins.str]] = None,
+        features: typing.Optional[typing.Sequence[typing.Union[DevContainerFeature, typing.Dict[builtins.str, typing.Any]]]] = None,
     ) -> None:
         '''(experimental) Constructor options for the DevContainer component.
 
@@ -178,6 +136,7 @@ class DevContainerOptions(_DevEnvironmentOptions_b10d89d1):
         :param ports: (experimental) An array of ports that should be exposed from the container.
         :param tasks: (experimental) An array of tasks that should be run when the container starts.
         :param vscode_extensions: (experimental) An array of extension IDs that specify the extensions that should be installed inside the container when it is created.
+        :param features: (experimental) An array of VSCode features that specify the features that should be installed inside the container when it is created.
 
         :see: https://github.com/microsoft/vscode-dev-containers/tree/master/containers/codespaces-linux
         :stability: experimental
@@ -188,6 +147,7 @@ class DevContainerOptions(_DevEnvironmentOptions_b10d89d1):
             check_type(argname="argument ports", value=ports, expected_type=type_hints["ports"])
             check_type(argname="argument tasks", value=tasks, expected_type=type_hints["tasks"])
             check_type(argname="argument vscode_extensions", value=vscode_extensions, expected_type=type_hints["vscode_extensions"])
+            check_type(argname="argument features", value=features, expected_type=type_hints["features"])
         self._values: typing.Dict[builtins.str, typing.Any] = {}
         if docker_image is not None:
             self._values["docker_image"] = docker_image
@@ -197,6 +157,8 @@ class DevContainerOptions(_DevEnvironmentOptions_b10d89d1):
             self._values["tasks"] = tasks
         if vscode_extensions is not None:
             self._values["vscode_extensions"] = vscode_extensions
+        if features is not None:
+            self._values["features"] = features
 
     @builtins.property
     def docker_image(self) -> typing.Optional[_DevEnvironmentDockerImage_4a8d8ffd]:
@@ -234,6 +196,15 @@ class DevContainerOptions(_DevEnvironmentOptions_b10d89d1):
         result = self._values.get("vscode_extensions")
         return typing.cast(typing.Optional[typing.List[builtins.str]], result)
 
+    @builtins.property
+    def features(self) -> typing.Optional[typing.List[DevContainerFeature]]:
+        '''(experimental) An array of VSCode features that specify the features that should be installed inside the container when it is created.
+
+        :stability: experimental
+        '''
+        result = self._values.get("features")
+        return typing.cast(typing.Optional[typing.List[DevContainerFeature]], result)
+
     def __eq__(self, rhs: typing.Any) -> builtins.bool:
         return isinstance(rhs, self.__class__) and rhs._values == self._values
 
@@ -246,6 +217,49 @@ class DevContainerOptions(_DevEnvironmentOptions_b10d89d1):
         )
 
 
+@jsii.interface(jsii_type="projen.vscode.IDevContainerEnvironment")
+class IDevContainerEnvironment(_IDevEnvironment_9a084622, typing_extensions.Protocol):
+    '''
+    :stability: experimental
+    '''
+
+    @jsii.member(jsii_name="addFeatures")
+    def add_features(self, *features: DevContainerFeature) -> None:
+        '''(experimental) Adds a list of VSCode features that should be automatically installed in the container.
+
+        :param features: featureName and version(optional default: latest).
+
+        :stability: experimental
+        '''
+        ...
+
+
+class _IDevContainerEnvironmentProxy(
+    jsii.proxy_for(_IDevEnvironment_9a084622), # type: ignore[misc]
+):
+    '''
+    :stability: experimental
+    '''
+
+    __jsii_type__: typing.ClassVar[str] = "projen.vscode.IDevContainerEnvironment"
+
+    @jsii.member(jsii_name="addFeatures")
+    def add_features(self, *features: DevContainerFeature) -> None:
+        '''(experimental) Adds a list of VSCode features that should be automatically installed in the container.
+
+        :param features: featureName and version(optional default: latest).
+
+        :stability: experimental
+        '''
+        if __debug__:
+            type_hints = typing.get_type_hints(_typecheckingstub__59f53078ad4f94636f7ad21968f318db42126c330e189eccadb0cb6fefb6bfd3)
+            check_type(argname="argument features", value=features, expected_type=typing.Tuple[type_hints["features"], ...]) # pyright: ignore [reportGeneralTypeIssues]
+        return typing.cast(None, jsii.invoke(self, "addFeatures", [*features]))
+
+# Adding a "__jsii_proxy_class__(): typing.Type" function to the interface
+typing.cast(typing.Any, IDevContainerEnvironment).__jsii_proxy_class__ = lambda : _IDevContainerEnvironmentProxy
+
+
 @jsii.enum(jsii_type="projen.vscode.InternalConsoleOptions")
 class InternalConsoleOptions(enum.Enum):
     '''(experimental) Controls the visibility of the VSCode Debug Console panel during a debugging session Source: https://code.visualstudio.com/docs/editor/debugging#_launchjson-attributes.
@@ -475,6 +489,28 @@ class VsCodeLaunchConfig(
             check_type(argname="argument vscode", value=vscode, expected_type=type_hints["vscode"])
         jsii.create(self.__class__, self, [vscode])
 
+    @jsii.member(jsii_name="addCommandInput")
+    def add_command_input(
+        self,
+        *,
+        command: builtins.str,
+        args: typing.Any = None,
+        id: builtins.str,
+    ) -> None:
+        '''(experimental) Adds an input variable with type ``command`` to ``.vscode/launch.json``.
+
+        See https://code.visualstudio.com/docs/editor/variables-reference#_input-variables for details.
+
+        :param command: 
+        :param args: 
+        :param id: 
+
+        :stability: experimental
+        '''
+        cfg = VsCodeLaunchCommandInputEntry(command=command, args=args, id=id)
+
+        return typing.cast(None, jsii.invoke(self, "addCommandInput", [cfg]))
+
     @jsii.member(jsii_name="addConfiguration")
     def add_configuration(
         self,
@@ -559,6 +595,66 @@ class VsCodeLaunchConfig(
 
         return typing.cast(None, jsii.invoke(self, "addConfiguration", [cfg]))
 
+    @jsii.member(jsii_name="addPickStringInput")
+    def add_pick_string_input(
+        self,
+        *,
+        description: builtins.str,
+        options: typing.Sequence[builtins.str],
+        default: typing.Optional[builtins.str] = None,
+        id: builtins.str,
+    ) -> None:
+        '''(experimental) Adds an input variable with type ``pickString`` to ``.vscode/launch.json``.
+
+        See https://code.visualstudio.com/docs/editor/variables-reference#_input-variables for details.
+
+        :param description: 
+        :param options: 
+        :param default: 
+        :param id: 
+
+        :stability: experimental
+        '''
+        cfg = VsCodeLaunchPickStringInputEntry(
+            description=description, options=options, default=default, id=id
+        )
+
+        return typing.cast(None, jsii.invoke(self, "addPickStringInput", [cfg]))
+
+    @jsii.member(jsii_name="addPromptStringInput")
+    def add_prompt_string_input(
+        self,
+        *,
+        description: builtins.str,
+        default: typing.Optional[builtins.str] = None,
+        password: typing.Optional[builtins.bool] = None,
+        id: builtins.str,
+    ) -> None:
+        '''(experimental) Adds an input variable with type ``promptString`` to ``.vscode/launch.json``.
+
+        See https://code.visualstudio.com/docs/editor/variables-reference#_input-variables for details.
+
+        :param description: 
+        :param default: 
+        :param password: 
+        :param id: 
+
+        :stability: experimental
+        '''
+        cfg = VsCodeLaunchPromptStringInputEntry(
+            description=description, default=default, password=password, id=id
+        )
+
+        return typing.cast(None, jsii.invoke(self, "addPromptStringInput", [cfg]))
+
+    @builtins.property
+    @jsii.member(jsii_name="file")
+    def file(self) -> _JsonFile_fa8164db:
+        '''
+        :stability: experimental
+        '''
+        return typing.cast(_JsonFile_fa8164db, jsii.get(self, "file"))
+
 
 @jsii.data_type(
     jsii_type="projen.vscode.VsCodeLaunchConfigurationEntry",
@@ -922,6 +1018,225 @@ class VsCodeLaunchConfigurationEntry:
         )
 
 
+@jsii.data_type(
+    jsii_type="projen.vscode.VsCodeLaunchInputEntry",
+    jsii_struct_bases=[],
+    name_mapping={"id": "id"},
+)
+class VsCodeLaunchInputEntry:
+    def __init__(self, *, id: builtins.str) -> None:
+        '''(experimental) Base options for a 'VsCodeLaunchInputEntry' Source: https://code.visualstudio.com/docs/editor/variables-reference#_input-variables.
+
+        :param id: 
+
+        :stability: experimental
+        '''
+        if __debug__:
+            type_hints = typing.get_type_hints(_typecheckingstub__e841e7e4ef3640a56bd02bfa85775b91ed62dd7fc9a8fcd482e487b75d2d4baa)
+            check_type(argname="argument id", value=id, expected_type=type_hints["id"])
+        self._values: typing.Dict[builtins.str, typing.Any] = {
+            "id": id,
+        }
+
+    @builtins.property
+    def id(self) -> builtins.str:
+        '''
+        :stability: experimental
+        '''
+        result = self._values.get("id")
+        assert result is not None, "Required property 'id' is missing"
+        return typing.cast(builtins.str, result)
+
+    def __eq__(self, rhs: typing.Any) -> builtins.bool:
+        return isinstance(rhs, self.__class__) and rhs._values == self._values
+
+    def __ne__(self, rhs: typing.Any) -> builtins.bool:
+        return not (rhs == self)
+
+    def __repr__(self) -> str:
+        return "VsCodeLaunchInputEntry(%s)" % ", ".join(
+            k + "=" + repr(v) for k, v in self._values.items()
+        )
+
+
+@jsii.data_type(
+    jsii_type="projen.vscode.VsCodeLaunchPickStringInputEntry",
+    jsii_struct_bases=[VsCodeLaunchInputEntry],
+    name_mapping={
+        "id": "id",
+        "description": "description",
+        "options": "options",
+        "default": "default",
+    },
+)
+class VsCodeLaunchPickStringInputEntry(VsCodeLaunchInputEntry):
+    def __init__(
+        self,
+        *,
+        id: builtins.str,
+        description: builtins.str,
+        options: typing.Sequence[builtins.str],
+        default: typing.Optional[builtins.str] = None,
+    ) -> None:
+        '''(experimental) Options for a 'VsCodeLaunchPickStringInputEntry' Source: https://code.visualstudio.com/docs/editor/variables-reference#_input-variables.
+
+        :param id: 
+        :param description: 
+        :param options: 
+        :param default: 
+
+        :stability: experimental
+        '''
+        if __debug__:
+            type_hints = typing.get_type_hints(_typecheckingstub__1efc4534d421a4d5a6d1ee19a7af423129f26ea3d3e365b0cbd5d316713e44b2)
+            check_type(argname="argument id", value=id, expected_type=type_hints["id"])
+            check_type(argname="argument description", value=description, expected_type=type_hints["description"])
+            check_type(argname="argument options", value=options, expected_type=type_hints["options"])
+            check_type(argname="argument default", value=default, expected_type=type_hints["default"])
+        self._values: typing.Dict[builtins.str, typing.Any] = {
+            "id": id,
+            "description": description,
+            "options": options,
+        }
+        if default is not None:
+            self._values["default"] = default
+
+    @builtins.property
+    def id(self) -> builtins.str:
+        '''
+        :stability: experimental
+        '''
+        result = self._values.get("id")
+        assert result is not None, "Required property 'id' is missing"
+        return typing.cast(builtins.str, result)
+
+    @builtins.property
+    def description(self) -> builtins.str:
+        '''
+        :stability: experimental
+        '''
+        result = self._values.get("description")
+        assert result is not None, "Required property 'description' is missing"
+        return typing.cast(builtins.str, result)
+
+    @builtins.property
+    def options(self) -> typing.List[builtins.str]:
+        '''
+        :stability: experimental
+        '''
+        result = self._values.get("options")
+        assert result is not None, "Required property 'options' is missing"
+        return typing.cast(typing.List[builtins.str], result)
+
+    @builtins.property
+    def default(self) -> typing.Optional[builtins.str]:
+        '''
+        :stability: experimental
+        '''
+        result = self._values.get("default")
+        return typing.cast(typing.Optional[builtins.str], result)
+
+    def __eq__(self, rhs: typing.Any) -> builtins.bool:
+        return isinstance(rhs, self.__class__) and rhs._values == self._values
+
+    def __ne__(self, rhs: typing.Any) -> builtins.bool:
+        return not (rhs == self)
+
+    def __repr__(self) -> str:
+        return "VsCodeLaunchPickStringInputEntry(%s)" % ", ".join(
+            k + "=" + repr(v) for k, v in self._values.items()
+        )
+
+
+@jsii.data_type(
+    jsii_type="projen.vscode.VsCodeLaunchPromptStringInputEntry",
+    jsii_struct_bases=[VsCodeLaunchInputEntry],
+    name_mapping={
+        "id": "id",
+        "description": "description",
+        "default": "default",
+        "password": "password",
+    },
+)
+class VsCodeLaunchPromptStringInputEntry(VsCodeLaunchInputEntry):
+    def __init__(
+        self,
+        *,
+        id: builtins.str,
+        description: builtins.str,
+        default: typing.Optional[builtins.str] = None,
+        password: typing.Optional[builtins.bool] = None,
+    ) -> None:
+        '''(experimental) Options for a 'VsCodeLaunchPromptStringInputEntry' Source: https://code.visualstudio.com/docs/editor/variables-reference#_input-variables.
+
+        :param id: 
+        :param description: 
+        :param default: 
+        :param password: 
+
+        :stability: experimental
+        '''
+        if __debug__:
+            type_hints = typing.get_type_hints(_typecheckingstub__fd9f5a97332f0e03a451cdcfdd927992c1488c8dafe0f46761677f3cb7f195f7)
+            check_type(argname="argument id", value=id, expected_type=type_hints["id"])
+            check_type(argname="argument description", value=description, expected_type=type_hints["description"])
+            check_type(argname="argument default", value=default, expected_type=type_hints["default"])
+            check_type(argname="argument password", value=password, expected_type=type_hints["password"])
+        self._values: typing.Dict[builtins.str, typing.Any] = {
+            "id": id,
+            "description": description,
+        }
+        if default is not None:
+            self._values["default"] = default
+        if password is not None:
+            self._values["password"] = password
+
+    @builtins.property
+    def id(self) -> builtins.str:
+        '''
+        :stability: experimental
+        '''
+        result = self._values.get("id")
+        assert result is not None, "Required property 'id' is missing"
+        return typing.cast(builtins.str, result)
+
+    @builtins.property
+    def description(self) -> builtins.str:
+        '''
+        :stability: experimental
+        '''
+        result = self._values.get("description")
+        assert result is not None, "Required property 'description' is missing"
+        return typing.cast(builtins.str, result)
+
+    @builtins.property
+    def default(self) -> typing.Optional[builtins.str]:
+        '''
+        :stability: experimental
+        '''
+        result = self._values.get("default")
+        return typing.cast(typing.Optional[builtins.str], result)
+
+    @builtins.property
+    def password(self) -> typing.Optional[builtins.bool]:
+        '''
+        :stability: experimental
+        '''
+        result = self._values.get("password")
+        return typing.cast(typing.Optional[builtins.bool], result)
+
+    def __eq__(self, rhs: typing.Any) -> builtins.bool:
+        return isinstance(rhs, self.__class__) and rhs._values == self._values
+
+    def __ne__(self, rhs: typing.Any) -> builtins.bool:
+        return not (rhs == self)
+
+    def __repr__(self) -> str:
+        return "VsCodeLaunchPromptStringInputEntry(%s)" % ", ".join(
+            k + "=" + repr(v) for k, v in self._values.items()
+        )
+
+
 class VsCodeRecommendedExtensions(
     _Component_2b0ad27f,
     metaclass=jsii.JSIIMeta,
@@ -971,6 +1286,14 @@ class VsCodeRecommendedExtensions(
             check_type(argname="argument extensions", value=extensions, expected_type=typing.Tuple[type_hints["extensions"], ...]) # pyright: ignore [reportGeneralTypeIssues]
         return typing.cast(None, jsii.invoke(self, "addUnwantedRecommendations", [*extensions]))
 
+    @builtins.property
+    @jsii.member(jsii_name="file")
+    def file(self) -> _JsonFile_fa8164db:
+        '''
+        :stability: experimental
+        '''
+        return typing.cast(_JsonFile_fa8164db, jsii.get(self, "file"))
+
 
 class VsCodeSettings(
     _Component_2b0ad27f,
@@ -1034,54 +1357,236 @@ class VsCodeSettings(
             check_type(argname="argument languages", value=languages, expected_type=type_hints["languages"])
         return typing.cast(None, jsii.invoke(self, "addSettings", [settings, languages]))
 
+    @builtins.property
+    @jsii.member(jsii_name="file")
+    def file(self) -> _JsonFile_fa8164db:
+        '''
+        :stability: experimental
+        '''
+        return typing.cast(_JsonFile_fa8164db, jsii.get(self, "file"))
+
+
+@jsii.implements(IDevContainerEnvironment)
+class DevContainer(
+    _Component_2b0ad27f,
+    metaclass=jsii.JSIIMeta,
+    jsii_type="projen.vscode.DevContainer",
+):
+    '''(experimental) A development environment running VSCode in a container;
+
+    used by GitHub
+    codespaces.
+
+    :stability: experimental
+    '''
+
+    def __init__(
+        self,
+        project: _Project_57d89203,
+        *,
+        features: typing.Optional[typing.Sequence[typing.Union[DevContainerFeature, typing.Dict[builtins.str, typing.Any]]]] = None,
+        docker_image: typing.Optional[_DevEnvironmentDockerImage_4a8d8ffd] = None,
+        ports: typing.Optional[typing.Sequence[builtins.str]] = None,
+        tasks: typing.Optional[typing.Sequence[_Task_9fa875b6]] = None,
+        vscode_extensions: typing.Optional[typing.Sequence[builtins.str]] = None,
+    ) -> None:
+        '''
+        :param project: -
+        :param features: (experimental) An array of VSCode features that specify the features that should be installed inside the container when it is created.
+        :param docker_image: (experimental) A Docker image or Dockerfile for the container.
+        :param ports: (experimental) An array of ports that should be exposed from the container.
+        :param tasks: (experimental) An array of tasks that should be run when the container starts.
+        :param vscode_extensions: (experimental) An array of extension IDs that specify the extensions that should be installed inside the container when it is created.
+
+        :stability: experimental
+        '''
+        if __debug__:
+            type_hints = typing.get_type_hints(_typecheckingstub__50747b0b39db339de569d0b44aea84c4de3c0d9d7b268479f1a3852dfb712c0b)
+            check_type(argname="argument project", value=project, expected_type=type_hints["project"])
+        options = DevContainerOptions(
+            features=features,
+            docker_image=docker_image,
+            ports=ports,
+            tasks=tasks,
+            vscode_extensions=vscode_extensions,
+        )
+
+        jsii.create(self.__class__, self, [project, options])
+
+    @jsii.member(jsii_name="addDockerImage")
+    def add_docker_image(self, image: _DevEnvironmentDockerImage_4a8d8ffd) -> None:
+        '''(experimental) Add a custom Docker image or Dockerfile for the container.
+
+        :param image: -
+
+        :stability: experimental
+        '''
+        if __debug__:
+            type_hints = typing.get_type_hints(_typecheckingstub__24cf7d35da522ed5017c6db46a2565a5b5a09fbffc1dafabdf917aa798fea91d)
+            check_type(argname="argument image", value=image, expected_type=type_hints["image"])
+        return typing.cast(None, jsii.invoke(self, "addDockerImage", [image]))
+
+    @jsii.member(jsii_name="addFeatures")
+    def add_features(self, *features: DevContainerFeature) -> None:
+        '''(experimental) Adds a list of VSCode features that should be automatically installed in the container.
+
+        :param features: featureName and version(optional default: latest).
+
+        :stability: experimental
+        '''
+        if __debug__:
+            type_hints = typing.get_type_hints(_typecheckingstub__6ed3f741b9b118a06d8b8f3508504c5ecb8bcbfc9c285b78e0382ac0aa4186c7)
+            check_type(argname="argument features", value=features, expected_type=typing.Tuple[type_hints["features"], ...]) # pyright: ignore [reportGeneralTypeIssues]
+        return typing.cast(None, jsii.invoke(self, "addFeatures", [*features]))
+
+    @jsii.member(jsii_name="addPorts")
+    def add_ports(self, *ports: builtins.str) -> None:
+        '''(experimental) Adds ports that should be exposed (forwarded) from the container.
+
+        :param ports: The new ports.
+
+        :stability: experimental
+        '''
+        if __debug__:
+            type_hints = typing.get_type_hints(_typecheckingstub__3898b547dc05a8cd9427e24743e3ec87b3108cfb90befb94e77c24e156c3559d)
+            check_type(argname="argument ports", value=ports, expected_type=typing.Tuple[type_hints["ports"], ...]) # pyright: ignore [reportGeneralTypeIssues]
+        return typing.cast(None, jsii.invoke(self, "addPorts", [*ports]))
+
+    @jsii.member(jsii_name="addTasks")
+    def add_tasks(self, *tasks: _Task_9fa875b6) -> None:
+        '''(experimental) Adds tasks to run when the container starts.
+
+        Tasks will be run in sequence.
+
+        :param tasks: The new tasks.
+
+        :stability: experimental
+        '''
+        if __debug__:
+            type_hints = typing.get_type_hints(_typecheckingstub__4eccb3fb9f6eca40de5ffa59e4b75a85830e1113729bd9b08dc7e4128c2bfde0)
+            check_type(argname="argument tasks", value=tasks, expected_type=typing.Tuple[type_hints["tasks"], ...]) # pyright: ignore [reportGeneralTypeIssues]
+        return typing.cast(None, jsii.invoke(self, "addTasks", [*tasks]))
+
+    @jsii.member(jsii_name="addVscodeExtensions")
+    def add_vscode_extensions(self, *extensions: builtins.str) -> None:
+        '''(experimental) Adds a list of VSCode extensions that should be automatically installed in the container.
+
+        :param extensions: The extension IDs.
+
+        :stability: experimental
+        '''
+        if __debug__:
+            type_hints = typing.get_type_hints(_typecheckingstub__c13b0cafe05fa1eb28182ec3465e43421a4aa7c7e44e035db34dd4cdbce24537)
+            check_type(argname="argument extensions", value=extensions, expected_type=typing.Tuple[type_hints["extensions"], ...]) # pyright: ignore [reportGeneralTypeIssues]
+        return typing.cast(None, jsii.invoke(self, "addVscodeExtensions", [*extensions]))
+
+    @builtins.property
+    @jsii.member(jsii_name="config")
+    def config(self) -> typing.Any:
+        '''(experimental) Direct access to the devcontainer configuration (escape hatch).
+
+        :stability: experimental
+        '''
+        return typing.cast(typing.Any, jsii.get(self, "config"))
+
+
+@jsii.data_type(
+    jsii_type="projen.vscode.VsCodeLaunchCommandInputEntry",
+    jsii_struct_bases=[VsCodeLaunchInputEntry],
+    name_mapping={"id": "id", "command": "command", "args": "args"},
+)
+class VsCodeLaunchCommandInputEntry(VsCodeLaunchInputEntry):
+    def __init__(
+        self,
+        *,
+        id: builtins.str,
+        command: builtins.str,
+        args: typing.Any = None,
+    ) -> None:
+        '''(experimental) Options for a 'VsCodeLaunchCommandInputEntry' Source: https://code.visualstudio.com/docs/editor/variables-reference#_input-variables.
+
+        :param id: 
+        :param command: 
+        :param args: 
+
+        :stability: experimental
+        '''
+        if __debug__:
+            type_hints = typing.get_type_hints(_typecheckingstub__3572ac292b4bd9b3dc808ba9caa4797adcb64703144da90f4ad010fdedc62a30)
+            check_type(argname="argument id", value=id, expected_type=type_hints["id"])
+            check_type(argname="argument command", value=command, expected_type=type_hints["command"])
+            check_type(argname="argument args", value=args, expected_type=type_hints["args"])
+        self._values: typing.Dict[builtins.str, typing.Any] = {
+            "id": id,
+            "command": command,
+        }
+        if args is not None:
+            self._values["args"] = args
+
+    @builtins.property
+    def id(self) -> builtins.str:
+        '''
+        :stability: experimental
+        '''
+        result = self._values.get("id")
+        assert result is not None, "Required property 'id' is missing"
+        return typing.cast(builtins.str, result)
+
+    @builtins.property
+    def command(self) -> builtins.str:
+        '''
+        :stability: experimental
+        '''
+        result = self._values.get("command")
+        assert result is not None, "Required property 'command' is missing"
+        return typing.cast(builtins.str, result)
+
+    @builtins.property
+    def args(self) -> typing.Any:
+        '''
+        :stability: experimental
+        '''
+        result = self._values.get("args")
+        return typing.cast(typing.Any, result)
+
+    def __eq__(self, rhs: typing.Any) -> builtins.bool:
+        return isinstance(rhs, self.__class__) and rhs._values == self._values
+
+    def __ne__(self, rhs: typing.Any) -> builtins.bool:
+        return not (rhs == self)
+
+    def __repr__(self) -> str:
+        return "VsCodeLaunchCommandInputEntry(%s)" % ", ".join(
+            k + "=" + repr(v) for k, v in self._values.items()
+        )
+
 
 __all__ = [
     "Console",
     "DevContainer",
+    "DevContainerFeature",
     "DevContainerOptions",
+    "IDevContainerEnvironment",
     "InternalConsoleOptions",
     "Presentation",
     "ServerReadyAction",
     "VsCode",
+    "VsCodeLaunchCommandInputEntry",
     "VsCodeLaunchConfig",
     "VsCodeLaunchConfigurationEntry",
+    "VsCodeLaunchInputEntry",
+    "VsCodeLaunchPickStringInputEntry",
+    "VsCodeLaunchPromptStringInputEntry",
     "VsCodeRecommendedExtensions",
     "VsCodeSettings",
 ]
 
 publication.publish()
 
-def _typecheckingstub__50747b0b39db339de569d0b44aea84c4de3c0d9d7b268479f1a3852dfb712c0b(
-    project: _Project_57d89203,
+def _typecheckingstub__5a7f73b2b024ecc62d8491484836d55c16fc65fe902d48c89e27920fb1de29e7(
     *,
-    docker_image: typing.Optional[_DevEnvironmentDockerImage_4a8d8ffd] = None,
-    ports: typing.Optional[typing.Sequence[builtins.str]] = None,
-    tasks: typing.Optional[typing.Sequence[_Task_9fa875b6]] = None,
-    vscode_extensions: typing.Optional[typing.Sequence[builtins.str]] = None,
-) -> None:
-    """Type checking stubs"""
-    pass
-
-def _typecheckingstub__24cf7d35da522ed5017c6db46a2565a5b5a09fbffc1dafabdf917aa798fea91d(
-    image: _DevEnvironmentDockerImage_4a8d8ffd,
-) -> None:
-    """Type checking stubs"""
-    pass
-
-def _typecheckingstub__3898b547dc05a8cd9427e24743e3ec87b3108cfb90befb94e77c24e156c3559d(
-    *ports: builtins.str,
-) -> None:
-    """Type checking stubs"""
-    pass
-
-def _typecheckingstub__4eccb3fb9f6eca40de5ffa59e4b75a85830e1113729bd9b08dc7e4128c2bfde0(
-    *tasks: _Task_9fa875b6,
-) -> None:
-    """Type checking stubs"""
-    pass
-
-def _typecheckingstub__c13b0cafe05fa1eb28182ec3465e43421a4aa7c7e44e035db34dd4cdbce24537(
-    *extensions: builtins.str,
+    name: builtins.str,
+    version: typing.Optional[builtins.str] = None,
 ) -> None:
     """Type checking stubs"""
     pass
@@ -1092,6 +1597,13 @@ def _typecheckingstub__b22d9ca6de29afcacfab2404c9835e6e3021ec640c49dfdad9bfb651e
     ports: typing.Optional[typing.Sequence[builtins.str]] = None,
     tasks: typing.Optional[typing.Sequence[_Task_9fa875b6]] = None,
     vscode_extensions: typing.Optional[typing.Sequence[builtins.str]] = None,
+    features: typing.Optional[typing.Sequence[typing.Union[DevContainerFeature, typing.Dict[builtins.str, typing.Any]]]] = None,
+) -> None:
+    """Type checking stubs"""
+    pass
+
+def _typecheckingstub__59f53078ad4f94636f7ad21968f318db42126c330e189eccadb0cb6fefb6bfd3(
+    *features: DevContainerFeature,
 ) -> None:
     """Type checking stubs"""
     pass
@@ -1155,6 +1667,33 @@ def _typecheckingstub__76028c8d6272b5133841fb2049bcddc9a00879405477a0aa27f058f8a
     """Type checking stubs"""
     pass
 
+def _typecheckingstub__e841e7e4ef3640a56bd02bfa85775b91ed62dd7fc9a8fcd482e487b75d2d4baa(
+    *,
+    id: builtins.str,
+) -> None:
+    """Type checking stubs"""
+    pass
+
+def _typecheckingstub__1efc4534d421a4d5a6d1ee19a7af423129f26ea3d3e365b0cbd5d316713e44b2(
+    *,
+    id: builtins.str,
+    description: builtins.str,
+    options: typing.Sequence[builtins.str],
+    default: typing.Optional[builtins.str] = None,
+) -> None:
+    """Type checking stubs"""
+    pass
+
+def _typecheckingstub__fd9f5a97332f0e03a451cdcfdd927992c1488c8dafe0f46761677f3cb7f195f7(
+    *,
+    id: builtins.str,
+    description: builtins.str,
+    default: typing.Optional[builtins.str] = None,
+    password: typing.Optional[builtins.bool] = None,
+) -> None:
+    """Type checking stubs"""
+    pass
+
 def _typecheckingstub__e672107e647965d3df7010fe525812b0c15f85ddada55b01f48ebe2c3d565725(
     vscode: VsCode,
 ) -> None:
@@ -1193,3 +1732,54 @@ def _typecheckingstub__fa3e27ee6b180fccd6eab5d9670c7024eb3f8067cb08859636119e69f
 ) -> None:
     """Type checking stubs"""
     pass
+
+def _typecheckingstub__50747b0b39db339de569d0b44aea84c4de3c0d9d7b268479f1a3852dfb712c0b(
+    project: _Project_57d89203,
+    *,
+    features: typing.Optional[typing.Sequence[typing.Union[DevContainerFeature, typing.Dict[builtins.str, typing.Any]]]] = None,
+    docker_image: typing.Optional[_DevEnvironmentDockerImage_4a8d8ffd] = None,
+    ports: typing.Optional[typing.Sequence[builtins.str]] = None,
+    tasks: typing.Optional[typing.Sequence[_Task_9fa875b6]] = None,
+    vscode_extensions: typing.Optional[typing.Sequence[builtins.str]] = None,
+) -> None:
+    """Type checking stubs"""
+    pass
+
+def _typecheckingstub__24cf7d35da522ed5017c6db46a2565a5b5a09fbffc1dafabdf917aa798fea91d(
+    image: _DevEnvironmentDockerImage_4a8d8ffd,
+) -> None:
+    """Type checking stubs"""
+    pass
+
+def _typecheckingstub__6ed3f741b9b118a06d8b8f3508504c5ecb8bcbfc9c285b78e0382ac0aa4186c7(
+    *features: DevContainerFeature,
+) -> None:
+    """Type checking stubs"""
+    pass
+
+def _typecheckingstub__3898b547dc05a8cd9427e24743e3ec87b3108cfb90befb94e77c24e156c3559d(
+    *ports: builtins.str,
+) -> None:
+    """Type checking stubs"""
+    pass
+
+def _typecheckingstub__4eccb3fb9f6eca40de5ffa59e4b75a85830e1113729bd9b08dc7e4128c2bfde0(
+    *tasks: _Task_9fa875b6,
+) -> None:
+    """Type checking stubs"""
+    pass
+
+def _typecheckingstub__c13b0cafe05fa1eb28182ec3465e43421a4aa7c7e44e035db34dd4cdbce24537(
+    *extensions: builtins.str,
+) -> None:
+    """Type checking stubs"""
+    pass
+
+def _typecheckingstub__3572ac292b4bd9b3dc808ba9caa4797adcb64703144da90f4ad010fdedc62a30(
+    *,
+    id: builtins.str,
+    command: builtins.str,
+    args: typing.Any = None,
+) -> None:
+    """Type checking stubs"""
+    pass
